#include "squidwall.h"

void setup(void);

/* fd of logfile */
int64 fd_log;

/* ip's and ports */
struct ip_address clamd_ip;
struct ip_address proxy_ip;
unsigned clamd_port;
unsigned clams_port;
unsigned proxy_port;

unsigned long maxsize;

/* destination websites for squid */
char *redir_badurl=0;
char *redir_badcont=0;
char *redir_baduser=0;
char *redir_badhost=0;
char *redir_virusfound=0;
char *redir_safesite=0;

/* misc directories */
char *hosts_dir=0;
char *users_dir=0;
char *filters_dir=0;

/* pointer to the infos from squid @ fd=0 */
char *squid_url=0;
char *squid_ip=0;
char *squid_host=0;
char *squid_user=0;
char *squid_method=0;

/* decoded infos */
stralloc ip;
stralloc url;
stralloc host;
stralloc user;
stralloc method;
stralloc redirsite; /* contains the side, which will be given to squid (without \n) */

/* filters, of current user and host */
array *cfilter_host;
array *ufilter_host;
array *cfilter_user;
array *ufilter_user;

/* read configuration environment variables */
void setup(void)
{
	int r;
	char *x;

	/* init the stralloc's */
	stralloc_init(&ip);
	stralloc_init(&url);
	stralloc_init(&host);
	stralloc_init(&user);
	stralloc_init(&method);
	stralloc_init(&redirsite);

	/* open logfile */
	x=env_get("LOG_FILE");
	if (!x) x=LOG_FILE;
	if (!io_appendfile(&fd_log, x)) panic("could not open logfile");

	/* virus warning url */
	redir_virusfound=env_get("REDIR_VIRUSFOUND");
	if (!redir_virusfound) redir_virusfound=REDIR_VIRUSFOUND;

	/* user is not permitted to use the proxy */
	redir_baduser=env_get("REDIR_BADUSER");
	if (!redir_baduser) redir_baduser=REDIR_BADUSER;

	/* host is not permitted to use the proxy */
	redir_badhost=env_get("REDIR_BADHOST");
	if (!redir_badhost) redir_badhost=REDIR_BADHOST;

	/* url is not okay */
	redir_badurl=env_get("REDIR_BADURL");
	if (!redir_badurl) redir_badurl=REDIR_BADURL;

	/* content is not okay */
	redir_badcont=env_get("REDIR_BADCONT");
	if (!redir_badcont) redir_badhost=REDIR_BADCONT;

	/* this url is can't be denied */
	redir_safesite=env_get("REDIR_SAFESITE");
	if (!redir_safesite) redir_safesite=REDIR_SAFESITE;

	/* directory for the userfiles */
	users_dir=env_get("USERS_DIR");
	if (!users_dir) users_dir=USERS_DIR;

	/* directory for the hostfiles */
	hosts_dir=env_get("HOSTS_DIR");
	if (!hosts_dir) hosts_dir=HOSTS_DIR;

	/* directory for the filterfiles */
	filters_dir=env_get("FILTERS_DIR");
	if (!filters_dir) filters_dir=FILTERS_DIR;

	/* read clamd configuration */
	x=env_get("CLAMD");
	if (!x) x=CLAMD;
	r=ip4_scan(x, (char*)&clamd_ip);
	if (!r) panic("can't get ip of $CLAMD!");
	r=scan_uint(x+r+1, &clamd_port);
	if (!r) panic("can't get port of $CLAMD!");
	
	/* read proxy configuration */
	x=env_get("PROXY");
	if (!x) x=PROXY;
	r=ip4_scan(x, (char*)&proxy_ip);
	if (!r) panic("can't get ip of $PROXY!");
	r=scan_uint(x+r+1, &proxy_port);
	if (!r) panic("can't get port of $PROXY!");

	/* read max filesize for clamd virus scans */
	x=env_get("MAX_SIZE");
	if (!x) {
		maxsize=MAX_SIZE;
	} else {
		r=scan_ulong(x, &maxsize);
		if (!r) panic("can't read environment $MAX_SIZE!");
	}

	/* level of debugging */
#if DEBUG_LEVEL
	x=env_get("DEBUG_LEVEL");
	if (!x) {
		debug_level=DEBUG_LEVEL;
	} else {
		r=scan_ulong(x, &debug_level);
		if (!r) panic("can't read environment $DEBUG_LEVEL!");
	}
#endif

	return;
}

int main(int argc, char **argv)
{
	unsigned int i;
	stralloc sa;
	int r, flags;

	stralloc_init(&sa);
	setup();

	if (!io_fd(0)) return 111;
	if (!io_fd(1)) return 111;

	for (;;) {
		int state=0;

		/* free old stuff */
		if (!stralloc_copys(&sa, "")) die_nomem();
		if (!stralloc_copys(&ip, "")) die_nomem();
		if (!stralloc_copys(&url, "")) die_nomem();
		if (!stralloc_copys(&host, "")) die_nomem();
		if (!stralloc_copys(&user, "")) die_nomem();
		if (!stralloc_copys(&method, "")) die_nomem();
		if (!stralloc_copys(&redirsite, "")) die_nomem();

		for (;;) {
			io_wantread(0); io_wait(); io_dontwantread(0);
			r=buffer_getline_sa(buffer_0, &sa);
			if (r == 0) return 0;
			if (r == 1) break;
		}
		sa.s[sa.len-1]=0; /* remove the newline */

		/* format: url ip/host user method
		 * sample1: http://www.mcmilk.de/ 192.168.100.10/aspire mcmilk GET
		 * sample2: www.mcmilk.de:443 192.168.100.10/aspire mcmilk GET
		 * state=0 url=http://www.mcmilk.de/
		 * state=1 ip=192.168.100.10
		 * state=2 host=aspire
		 * state=3 user=root
		 * state=4 method=GET
		 */

		squid_url=sa.s;
		for (i=0; i<=sa.len; i++) {
			switch (state) {
			case 0:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=1; squid_ip=sa.s+i+1;
			}
			break;
			case 1:
			if (sa.s[i] == '/') {
				sa.s[i]=0; state=2; squid_host=sa.s+i+1;
			}
			break;
			case 2:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=3; squid_user=sa.s+i+1;
			}
			break;
			case 3:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=4; squid_method=sa.s+i+1;
			}
			break;
			} /* switch() */
		}

		if (state != 4) {
			log_headline("error");
			log_s("incorrect input from squid!\n");
			log_flush();
			buffer_putsaflush(buffer_1, &sa); /* return it to stdout */
			continue;                         /* and ignore it! */
		}
		
		if (str_start(squid_url, redir_safesite)) { goto out; }

		/* squid gives us urlencoded things - we decode them first! */
		if (!scan_to_sa(scan_urlencoded,  squid_ip, &ip)) die_nomem();
		if (!scan_to_sa(scan_urlencoded2, squid_url, &url)) die_nomem();
		if (!scan_to_sa(scan_urlencoded,  squid_host, &host)) die_nomem();
		if (!scan_to_sa(scan_urlencoded2, squid_user, &user)) die_nomem();
		if (!scan_to_sa(scan_urlencoded,  squid_method, &method)) die_nomem();
		ip.s[ip.len]=0;
		url.s[url.len]=0;
		host.s[host.len]=0;
		user.s[user.len]=0;
		method.s[method.len]=0;

		/* we check in this order:
		 *
		 * 1. is the user okay ?
		 * 2. is the host/ip okay ?
		 * 3. look for bad url's
		 * 4. check for viruses and/or bad content (if wanted)
		 */

		flags=0;
		cfilter_host=0; ufilter_host=0;
		cfilter_user=0; ufilter_user=0;

		/* 1. is the user okay ? */
		flags |= check_acl(TYPE_USER);
		if (isset(flags, ACL_DENY)) {
			redir_site(TYPE_USER); goto out;
		}

		/* 2. is the host/ip okay ? */
		flags |= check_acl(TYPE_HOST);
		if (isset(flags, ACL_DENY)) {
			redir_site(TYPE_HOST); goto out;
		}

		/* 3. check the url */
		flags |= check_url();
		if (isset(flags, ACL_DENY)) {
			redir_site(TYPE_URL);
			goto out;
		}

		/* 4. should we scan for viruses or bad content ? */
		if (isset(flags, ACL_FILTER)) {
			check_cont();
		}

		out:

		/* give squid an empty newline '\n' or the whole redirsite */
		io_wantwrite(1); io_wait(); io_dontwantwrite(1);
		buffer_putsa(buffer_1, &redirsite);
		buffer_puts(buffer_1, "\n");
		debug(9, "-----------------------------------------------------");
		log_flush();
		while (buffer_flush(buffer_1) == -1);
	}

	return 0;
}
